#!/usr/bin/python3

import sys
import random

default = {
    "n": 10,
    "m": 20,
    "k": 5,
    "w": 10**4,
    "only_shortest": 0,
}

def cmdlinearg(name):
    for arg in sys.argv:
        if arg.startswith(name + "="):
            return arg.split("=")[1]
    return default[name]

def main():

    random.seed(int(sys.argv[-1]))
    n = int(cmdlinearg("n"))
    m = int(cmdlinearg("m"))
    k = int(cmdlinearg("k"))
    w = int(cmdlinearg("w"))
    only_shortest = int(cmdlinearg("only_shortest"))

    edges_seen = set()

    weights = [random.randint(1, w) for _ in range(n)]
    distances = [0] * n
    distances[0] = weights[0]

    edges = []

    # Spanning tree
    for i in range(1, n):
        j = random.randrange(0, i)
        edges_seen.add((i, j))
        edges_seen.add((j, i))
        edges.append((i+1, j+1))
        distances[j] = weights[j] + distances[i]

    # A bunch of more edges
    MAX_RETRIES = 40
    for _ in range(m-n+1):
        a, b = random.randrange(0, n), random.randrange(0, n)
        if a > b:
            (a, b) = (b, a)
        attempts = 0
        while a == b or (a, b) in edges_seen or (only_shortest and distances[b] != distances[a] + weights[b]):
            a, b = random.randrange(0, n), random.randrange(0, n)
            if a > b:
                (a, b) = (b, a)
            attempts += 1
            if attempts >= MAX_RETRIES:
                break
        if attempts >= MAX_RETRIES:
            break
        edges_seen.add((a, b))
        edges_seen.add((b, a))
        edges.append((a+1, b+1))
    
    print(n,len(edges),k)
    print(*weights)

    random.shuffle(edges)
    for e in edges:
        print(*e)
    
if __name__ == "__main__":
    main()