#!/usr/bin/env pypy3
import random
import os
import time
from math import gcd

SOLUTION = "per_nx"

MAX_N = 20000
MAX_X = 10000

case_num = 1


def output(description, messages, x):
    global case_num

    with open(f"secret/{case_num:02}-{description}.in", "wt") as f:
        print(f"{len(messages)} {x}", file = f)
        print(' '.join(map(str, messages)), file = f)

    case_num += 1

def random_output(description, n, x, min_size, max_size, prob_too_large):
    messages = [
        random.randint(min_size, max_size)
        if random.random() > prob_too_large
        else
        random.randint(max_size, MAX_X)
        for _ in range(n)
    ]
    output(description, messages, x)

def clean():
    for filename in os.listdir("secret"):
        if filename.endswith(".in") or filename.endswith(".ans"):
            os.remove(f"secret/{filename}")

def create_ans():
    slowest = 0.0

    for filename in os.listdir("secret"):
        if filename.endswith(".in"):
            ans_filename = filename.replace(".in", ".ans")
            start = time.time()
            os.system(f"../submissions/accepted/{SOLUTION} <secret/{filename} >secret/{ans_filename}")
            elapsed = time.time() - start
            if elapsed > slowest:
                slowest = elapsed

    print(f"Slowest = {slowest:.2} s")
            

def breakgreedy(data):
    x = data[0] + 2
    d = [x + 1 - y for y in data]
    d.extend([1, 1, 1, 1])
    output("breakgreedy", d, x)

    
def gcd_case(max_n=MAX_N, max_x=MAX_X):
    X = max_x
    N = min(max_n, 2*X+3)
    while True:
        a = (N-4)//2
        while gcd(a, N-4-a) != 1 and N-4-a <= X:
            a -= 1
        if N-4-a <= X: break
        N -= 1
    t = [1]*4 + [X+1-a]*(N-4-a) + [X+1-(N-4-a)]*a
    random.shuffle(t)
    output("gcd", t, X)
    return N


def non_reducible(X, MAX_N=MAX_N-4, multiples_of = 1, odd_count = False, add_one_extra = False):
    from collections import defaultdict
    def is_reducible(F, newx):
        for x in F:
            if newx % x: continue
            mul = 0
            tot = 0
            vals = 0
            done = False
            while True:
                mul += 1
                if x*mul > X or mul > 1+tot: break
                if F.get(x*mul, 0) == 0: continue
                for _ in range(F[x*mul]):
                    tot += mul
                    vals += 1
                    if tot.bit_length() < vals: return True
        return False
    freq = defaultdict(lambda: 0)
    vals = list([i for i in range(1, X+1) if i % multiples_of == 0])
    last = -1
    while vals:
        while vals:
            x = vals.pop()
            freq[x] += 1
            if not is_reducible(freq, x):
                vals.append(x)
                break
            freq[x] -= 1
            if freq[x] == 0: del freq[x]
    if odd_count and sum(f for f in freq.values()) % 2 == 0:
        freq[last] -= 1
    if add_one_extra:
        vals = list([i for i in range(1, X+1) if i % multiples_of != 0])
        while vals:
            x = vals.pop()
            freq[x] += 1
            if not is_reducible(freq, x): break
            freq[x] -= 1
            del freq[x]
    res = []
    for (x, f) in freq.items():
        res.extend([X+1-x]*f)
    random.shuffle(res)
    if len(res) > MAX_N: res = res[:MAX_N]
    return res

if __name__ == "__main__":
    clean()

    output("max", [1] * MAX_N, MAX_X)
    output("max", [1] * (MAX_N - 1), MAX_X)

    for seed in range(10):
        random.seed(1000+seed)
        n = random.randint(1, 100)
        random_output("random-small", n, MAX_X, 1, MAX_X, 0.0)

    for seed in range(4):
        random.seed(100 + seed)
        n = random.randint(MAX_N // 2, MAX_N)
        max_size = random.randint(5, MAX_X)
        min_size = random.randint(3, max_size)
        x = max_size

        random_output("random-large", n, x, min_size, max_size, 0.25 if seed % 4 == 0 else 0.0)

    for seed in range(2):
        random.seed(seed)
        random_output("random-max", MAX_N, MAX_X, 1, MAX_X, 0.0)

    # Various boundary corner cases
    output("corner", [5], 9)
    output("corner", [12], 9)
    output("corner", [4, 15], 9)
    output("corner", [2, 3, 1, 19], 8)
    output("corner", [2, 3, 4, 5, 9], 7)
    output("corner", [2, 3, 4, 5], 7)
    output("corner", [2, 3, 4], 7)
    output("corner", [2, 7], 7)
    output("corner", [2, 3, 4, 5, 6, 7, 8] * (MAX_N//7), 1)
    output("corner", [4857], 10000)
    output("corner", [5002, 4918], 10000)

    output("breakgreedy", [30, 10, 1, 42, 38, 41, 45, 47, 5, 11, 72, 34, 68, 37], 90)
    output("breakgreedy", [412, 24, 206, 400, 121, 147, 112, 69, 313, 230, 131, 418, 100], 554)
    output("breakgreedy", [33, 210, 153, 246, 134, 80, 113, 260, 381, 273, 362, 276, 202, 23, 88, 135, 120], 386)

    breakgreedy([100, 100, 75, 88, 37, 37, 88, 48, 87, 34, 88, 88, 57, 16, 74, 91])
    breakgreedy([8000, 8000, 7000, 7000, 7000] + [x+1 for x in range(100)])
    breakgreedy([8000, 8000, 7000, 7000, 7000] + [1] * 1000)

    output('breakwrongopt', [6263, 527, 3340, 3511, 6834, 9585, 3364, 10000, 5430, 3211, 5533, 217, 7089, 4851, 847, 2627, 4619, 3005], 10000)


    for X in range(MAX_X, MAX_X-4, -1):
        N = MAX_N
        for _ in range(4):
            N = gcd_case(max_x=X, max_n=N) - 1

    output("max-allvalues1", list(range(1, MAX_X+1)) + [1]*(MAX_N-MAX_X), MAX_X)
    t = [1]*4
    for x in range(1, MAX_X+1):
        t.extend([x]*((MAX_N-4)//MAX_X + (x <= (MAX_N-4) % MAX_X)))
    output("max-allvalues2", t, MAX_X)

    output("nonreducible", non_reducible(MAX_X) + [1]*4, MAX_X)
    
    t = [1]*5
    for x in range(3, MAX_X+1, 3):
        t.extend([MAX_X+1-x]*((MAX_N-5)//(MAX_X//3) + (x//3 <= (MAX_N-5) % (MAX_X//3))))
    output("max-mod3-plus-one", t, MAX_X)
    
    t = [1]*5
    for x in range(3, MAX_X+1, 3):
        t.extend([MAX_X+1-x]*((MAX_N-6)//(MAX_X//3) + (x//3 <= (MAX_N-6) % (MAX_X//3))))
    output("max-mod3-plus-one", t, MAX_X)
    
    output("nonreducible-mod3-plus-one",
           non_reducible(MAX_X, multiples_of = 3, odd_count = True, add_one_extra=True) + [1]*4,
           MAX_X)
    
    print("Running solutions...")
    create_ans()
